@@ -66,9 +66,9 @@ def get_region_connections_dict(world: "TTYDWorld") -> dict[tuple[str, str], typ
         ("Rogueport Sewers", "Pit of 100 Trials"):
             lambda state: StateLogic.pit(state, world.player),
         ("Rogueport", "Shadow Queen"):
-            lambda state: StateLogic.palace(state, world.player, world.options.goal_stars.value),
+            lambda state: StateLogic.palace(state, world, world.options.goal_stars.value),
         ("Rogueport", "Palace of Shadow"):
-            lambda state: StateLogic.palace(state, world.player, world.options.palace_stars.value),
+            lambda state: StateLogic.palace(state, world, world.options.palace_stars.value),
         ("Palace of Shadow", "Palace of Shadow (Post-Riddle Tower)"):
             lambda state: StateLogic.riddle_tower(state, world.player),
         ("Palace of Shadow (Post-Riddle Tower)", "Shadow Queen"):
@@ -85,6 +85,8 @@ def get_region_connections_dict(world: "TTYDWorld") -> dict[tuple[str, str], typ
             lambda state: StateLogic.westside(state, world.player),
         ("Rogueport (Westside)", "Glitzville"):
             lambda state: StateLogic.glitzville(state, world.player),
+        ("Rogueport (Westside)", "Rogueport Sewers Westside"):
+            lambda state: state.has("Paper Mode", world.player),
         ("Rogueport (Westside)", "Excess Express"):
             lambda state: StateLogic.excess_express(state, world.player),
         ("Excess Express", "Riverside Station"):
