@@ -1,16 +1,17 @@
 import logging
 import os
 
-from Fill import fill_restrictive, fast_fill
+from Fill import fill_restrictive
 from typing import List, Dict, ClassVar, Any, Set
 from settings import UserFilePath, Group
 from BaseClasses import Tutorial, ItemClassification, CollectionState, Item, Location
 from worlds.AutoWorld import WebWorld, World
 from .Data import starting_partners, limit_eight, stars, chapter_items, limited_location_ids, limit_pit, \
-    pit_exclusive_tattle_stars_required
+    pit_exclusive_tattle_stars_required, dazzle_counts, dazzle_location_names, star_locations
 from .Locations import all_locations, location_table, location_id_to_name, TTYDLocation, locationName_to_data, \
     get_locations_by_tags, get_vanilla_item_names, get_location_names, LocationData
-from .Options import Piecesanity, TTYDOptions, YoshiColor, StartingPartner, PitItems, LimitChapterEight, Goal
+from .Options import Piecesanity, TTYDOptions, YoshiColor, StartingPartner, PitItems, LimitChapterEight, Goal, \
+    DazzleRewards, StarShuffle
 from .Items import TTYDItem, itemList, item_table, ItemData, items_by_id
 from .Regions import create_regions, connect_regions, get_regions_dict, register_indirect_connections
 from .Rom import TTYDProcedurePatch, write_files
@@ -68,9 +69,10 @@ class TTYDSettings(Group):
 
 class TTYDWorld(World):
     """
-    Paper Mario: The Thousand-Year Door is a 2004 role-playing video game and the second game in the Paper Mario series following Paper Mario, and is part of the larger Mario franchise. 
-    In the game, when Mario and Princess Peach get involved in the search for a mystic treasure that holds great fortune, Peach is kidnapped by an alien group called the X-Nauts; 
-    Mario sets out to find the treasure and save the princess.
+    Paper Mario: The Thousand-Year Door is a quirky, turn-based RPG with a paper-craft twist.
+    Mario teams up with oddball allies to stop an ancient evil sealed behind a magical door.
+    Set in Rogueport, the game mixes platforming, puzzles, and witty, self-aware dialogue.
+    Battles play out on a stage with timed button presses and a live audience cheering you on.
     """
 
     from BaseUtils import get_archipelago_json
@@ -86,10 +88,11 @@ class TTYDWorld(World):
     settings: ClassVar[TTYDSettings]
     item_name_to_id = {name: data.id for name, data in item_table.items()}
     location_name_to_id = {loc_data.name: loc_data.id for loc_data in all_locations}
-    required_client_version = (0, 6, 0)
+    required_client_version = (0, 6, 2)
     disabled_locations: set
     excluded_regions: set
     items: List[TTYDItem]
+    star_pieces: List[TTYDItem]
     required_chapters: List[int]
     limited_chapters: List[int]
     limited_chapter_locations: Set[Location]
@@ -97,17 +100,34 @@ class TTYDWorld(World):
     limited_items: List[TTYDItem]
     limited_state: CollectionState = None
     locked_item_frequencies: Dict[str, int]
+    ut_can_gen_without_yaml = True
 
     def generate_early(self) -> None:
         self.disabled_locations = set()
         self.excluded_regions = set()
         self.items = []
+        self.star_pieces = []
         self.required_chapters = []
         self.limited_chapters = []
         self.limited_chapter_locations = set()
         self.limited_item_names = set()
         self.limited_items = []
         self.locked_item_frequencies = {}
+        # implementing yaml-less UT support
+        if hasattr(self.multiworld, "re_gen_passthrough"):
+            if self.game in self.multiworld.re_gen_passthrough:
+                slot_data = self.multiworld.re_gen_passthrough[self.game]
+                self.options.goal.value = slot_data["goal"]
+                self.options.goal_stars.value = slot_data["goal_stars"]
+                self.options.palace_stars.value = slot_data["palace_stars"]
+                self.options.pit_items.value = slot_data["pit_items"]
+                self.options.limit_chapter_logic.value = slot_data["limit_chapter_logic"]
+                self.options.limit_chapter_eight.value = slot_data["limit_chapter_eight"]
+                self.options.palace_skip.value = slot_data["palace_skip"]
+                self.options.open_westside.value = slot_data["westside"]
+                self.options.tattlesanity.value = slot_data["tattlesanity"]
+                self.options.disable_intermissions.value = slot_data["disable_intermissions"]
+                return
         if self.options.limit_chapter_eight and self.options.palace_skip:
             logging.warning(f"{self.player_name}'s has enabled both Palace Skip and Limit Chapter 8. "
                             f"Disabling the Limit Chapter 8 option due to incompatibility.")
@@ -116,7 +136,7 @@ class TTYDWorld(World):
             logging.warning(f"{self.player_name}'s has Bonetail as the goal with less than 5 stars required. "
                             f"Increasing number of goal stars to 5 for accessibility.")
             self.options.goal_stars.value = 5
-        if self.options.goal != Goal.option_shadow_queen and self.options.palace_stars > self.options.goal_stars:
+        if self.options.palace_stars > self.options.goal_stars:
             logging.warning(f"{self.player_name}'s has more palace stars required than goal stars. "
                             f"Reducing number of stars required to enter the palace of shadow for accessibility.")
             self.options.palace_stars.value = self.options.goal_stars.value
@@ -157,8 +177,14 @@ class TTYDWorld(World):
         if self.options.tattlesanity:
             self.limit_tattle_locations()
         self.lock_item_remove_from_pool("Rogueport Center: Goombella", starting_partners[self.options.starting_partner.value - 1])
-        if not self.options.star_shuffle:
+        if self.options.star_shuffle == StarShuffle.option_vanilla:
             self.lock_vanilla_items_remove_from_pool(get_locations_by_tags("star"))
+        elif self.options.star_shuffle == StarShuffle.option_stars_only:
+            locations = get_locations_by_tags("star")
+            items = [location.vanilla_item for location in locations]
+            self.multiworld.random.shuffle(items)
+            for i, location in enumerate(locations):
+                self.lock_item_remove_from_pool(location.name, items_by_id[items[i]].item_name)
         if self.options.goal == Goal.option_shadow_queen:
             self.lock_item("Shadow Queen", "Victory")
         if self.options.limit_chapter_eight:
@@ -169,7 +195,9 @@ class TTYDWorld(World):
                     self.lock_item_remove_from_pool(location.name, "Palace Key")
             self.lock_item_remove_from_pool("Palace of Shadow Gloomtail Room: Star Key", "Star Key")
         if self.options.palace_skip:
-            self.lock_vanilla_items_remove_from_pool(get_locations_by_tags(["riddle_tower", "palace"]))
+            self.locked_item_frequencies["Palace Key"] = 3
+            self.locked_item_frequencies["Palace Key (Tower)"] = 8
+            self.locked_item_frequencies["Star Key"] = 1
         if self.options.pit_items == PitItems.option_vanilla:
             self.lock_vanilla_items_remove_from_pool(get_locations_by_tags("pit_floor"))
         if self.options.piecesanity == Piecesanity.option_vanilla:
@@ -182,6 +210,10 @@ class TTYDWorld(World):
             self.lock_vanilla_items_remove_from_pool(get_locations_by_tags("shop"))
         if self.options.pit_items == PitItems.option_filler:
             self.lock_filler_items_remove_from_pool(get_locations_by_tags("pit_floor"))
+        if self.options.dazzle_rewards == DazzleRewards.option_vanilla:
+            self.lock_vanilla_items_remove_from_pool(get_locations_by_tags("dazzle"))
+        elif self.options.dazzle_rewards == DazzleRewards.option_filler:
+            self.lock_filler_items_remove_from_pool(get_locations_by_tags("dazzle"))
 
 
     def limit_tattle_locations(self):
@@ -211,28 +243,41 @@ class TTYDWorld(World):
         # First add in all progression items
         self.items = []
         self.limited_items = []
+        self.star_pieces = []
         self.limited_state = CollectionState(self.multiworld)
         required_items = []
         precollected = [item for item in itemList if item in self.multiworld.precollected_items[self.player]]
         added_items = 0
         for chapter in self.limited_chapters:
-            self.limited_item_names.update(chapter_items[chapter] + (stars[chapter - 1] if self.options.star_shuffle else []))
+            if chapter != 8:
+                self.limited_item_names.update(chapter_items[chapter] + ([stars[chapter - 1]] if self.options.star_shuffle == StarShuffle.option_all else []))
+            else:
+                self.limited_item_names.update(chapter_items[chapter])
         for item in [item for item in itemList if item.progression == ItemClassification.progression]:
             if item not in precollected:
-                frequency = max(item.frequency - self.locked_item_frequencies.get(item.id, 0), 0)
+                frequency = max(item.frequency - self.locked_item_frequencies.get(item.item_name, 0), 0)
+                if frequency == 0:
+                    continue
+                if self.locked_item_frequencies.get(item.item_name, 0) > 0:
+                    logging.info(f"{self.player_name}'s locked item {item.item_name} reduced its frequency from {item.frequency} to {frequency}.")
                 required_items += [item.item_name for _ in range(frequency)]
         for item_name in required_items:
             item = self.create_item(item_name)
             if item_name in self.limited_item_names:
                 self.limited_items.append(item)
             else:
-                self.limited_state.collect(item, prevent_sweep=True)
-                self.multiworld.itempool.append(item)
+                if item_name == "Star Piece":
+                    self.star_pieces.append(item)
+                else:
+                    self.limited_state.collect(item, prevent_sweep=True)
+                    self.multiworld.itempool.append(item)
             added_items += 1
 
         useful_items = []
         for item in [item for item in itemList if item.progression == ItemClassification.useful]:
-            frequency = max(item.frequency - self.locked_item_frequencies.get(item.id, 0), 0)
+            frequency = max(item.frequency - self.locked_item_frequencies.get(item.item_name, 0), 0)
+            if frequency == 0:
+                continue
             useful_items += [item.item_name for _ in range(frequency)]
 
         for item_name in useful_items:
@@ -244,14 +289,16 @@ class TTYDWorld(World):
         filler_items = []
         for item in itemList:
             if item.progression == ItemClassification.filler:
-                freq = item.frequency
-                if freq is None:
-                    freq = 1
+                frequency = max(item.frequency - self.locked_item_frequencies.get(item.item_name, 0), 0)
                 if self.options.tattlesanity:
-                    freq += 2
-                filler_items += [item.item_name for _ in range(freq)]
+                    frequency += 2
+                if frequency == 0:
+                    continue
+                filler_items += [item.item_name for _ in range(frequency)]
 
         remaining = len(self.multiworld.get_unfilled_locations(self.player)) - added_items
+        if len(filler_items) < remaining:
+            filler_items += [self.get_filler_item_name() for _ in range(remaining - len(filler_items))]
         for i in range(remaining):
             filler_item_name = self.multiworld.random.choice(filler_items)
             item = self.create_item(filler_item_name)
@@ -260,15 +307,34 @@ class TTYDWorld(World):
 
         if len(self.limited_chapter_locations) > 0:
             self.multiworld.random.shuffle(self.items)
-            for _ in range(len([location for location in self.limited_chapter_locations if location in self.multiworld.get_unfilled_locations(self.player)]) - len(self.limited_items)):
+            logging.info(f"{self.player_name}'s Number of limited locations: {len(self.limited_chapter_locations)}, number of items: {len(self.items)}, number of limited items: {len(self.limited_items)}")
+            item_count = len([location for location in self.limited_chapter_locations if location in self.multiworld.get_unfilled_locations(self.player)]) - len(self.limited_items)
+            for _ in range(min(item_count, len(self.items))):
                 self.limited_items.append(self.items.pop())
+            if item_count > len(self.limited_items):
+                logging.warning(f"{self.player_name}'s Not enough items to fill limited locations. {item_count} locations but only {len(self.limited_items)} items.")
+                logging.warning(f"Supplementing items with star pieces.")
+                for _ in range(item_count - len(self.limited_items)):
+                    self.limited_items.append(self.star_pieces.pop())
 
-        for item in self.items:
+        if self.options.dazzle_rewards != DazzleRewards.option_all:
+            limited_star_pieces = len([item for item in self.limited_items if item.name == "Star Piece"] + [location for location in self.limited_chapter_locations if location.item is not None and location.item.name == "Star Piece"])
+            logging.info(f"{self.player_name}'s Number of star pieces in limited locations: {limited_star_pieces}")
+            dazzle_locations = [self.get_location(location_name) for location_name in dazzle_location_names]
+            for i, location in enumerate(dazzle_locations):
+                if limited_star_pieces > (100 - dazzle_counts[i - 1]):
+                    self.limited_chapter_locations.add(location)
+                    if len(self.items) > 0:
+                        self.limited_items.append(self.create_item(self.get_filler_item_name()))
+                    else:
+                        self.limited_items.append(self.star_pieces.pop())
+
+        for item in self.items + self.star_pieces:
             self.multiworld.itempool.append(item)
 
     def pre_fill(self) -> None:
         _ = [self.limited_state.collect(location.item, prevent_sweep=True) for location in self.get_locations() if
-             location.item is not None and location.item.name not in stars]
+             location.item is not None and location.item.name not in stars and location.item.name != "Victory"]
         for chapter in self.limited_chapters:
             locations = [location for location in self.limited_chapter_locations if location.item is None and location.name in get_location_names(get_locations_by_tags(f"chapter_{chapter}"))]
             progressive_items = [item for item in self.limited_items if item.name in chapter_items[chapter]]
@@ -276,7 +342,7 @@ class TTYDWorld(World):
             self.multiworld.random.shuffle(progressive_items)
             self.multiworld.random.shuffle(locations)
             fill_restrictive(self.multiworld, self.limited_state, locations, progressive_items, single_player_placement=True, lock=True)
-        fast_fill(self.multiworld, self.limited_items, [location for location in self.limited_chapter_locations if location.item is None])
+        fill_restrictive(self.multiworld, self.limited_state, [location for location in self.limited_chapter_locations if location.item is None], self.limited_items, single_player_placement=True, lock=True)
 
     def set_rules(self) -> None:
         set_rules(self)
@@ -300,9 +366,13 @@ class TTYDWorld(World):
             "yoshi_color": self.options.yoshi_color.value,
             "westside": self.options.open_westside.value,
             "tattlesanity": self.options.tattlesanity.value,
+            "dazzle_rewards": self.options.dazzle_rewards.value,
+            "star_shuffle": self.options.star_shuffle.value,
             "disable_intermissions": self.options.disable_intermissions.value,
             "cutscene_skip": self.options.cutscene_skip.value,
             "death_link": self.options.death_link.value,
+            "piecesanity": self.options.piecesanity.value,
+            "shinesanity": self.options.shinesanity.value
         }
 
     def create_item(self, name: str) -> TTYDItem:
@@ -312,6 +382,7 @@ class TTYDWorld(World):
 
     def lock_item(self, location: str, item_name: str):
         item = self.create_item(item_name)
+        item.location = self.get_location(location)
         if location not in self.disabled_locations:
             self.get_location(location).place_locked_item(item)
 
@@ -320,7 +391,9 @@ class TTYDWorld(World):
             locations = [locations]
         for location in locations:
             if location.name not in self.disabled_locations:
-                self.get_location(location.name).place_locked_item(self.create_item(items_by_id[location.vanilla_item].item_name))
+                item = self.create_item(items_by_id[location.vanilla_item].item_name)
+                item.location = self.get_location(location.name)
+                self.get_location(location.name).place_locked_item(item)
             
     def lock_vanilla_items_remove_from_pool(self, locations: LocationData | List[LocationData]) -> None:
         if isinstance(locations, LocationData):
@@ -328,7 +401,9 @@ class TTYDWorld(World):
         for location in locations:
             self.locked_item_frequencies[items_by_id[location.vanilla_item].item_name] = self.locked_item_frequencies.get(items_by_id[location.vanilla_item].item_name, 0) + 1
             if location.name not in self.disabled_locations:
-                self.get_location(location.name).place_locked_item(self.create_item(items_by_id[location.vanilla_item].item_name))
+                item = self.create_item(items_by_id[location.vanilla_item].item_name)
+                item.location = self.get_location(location.name)
+                self.get_location(location.name).place_locked_item(item)
 
     def lock_filler_items_remove_from_pool(self, locations: LocationData | List[LocationData]) -> None:
         if isinstance(locations, LocationData):
@@ -337,11 +412,14 @@ class TTYDWorld(World):
             filler_item_name = self.get_filler_item_name()
             self.locked_item_frequencies[filler_item_name] = self.locked_item_frequencies.get(filler_item_name, 0) + 1
             if location.name not in self.disabled_locations:
-                self.get_location(location.name).place_locked_item(self.create_item(filler_item_name))
+                item = self.create_item(filler_item_name)
+                item.location = self.get_location(location.name)
+                self.get_location(location.name).place_locked_item(item)
 
     def lock_item_remove_from_pool(self, location: str, item_name: str):
         self.locked_item_frequencies[item_name] = self.locked_item_frequencies.get(item_name, 0) + 1
         item = self.create_item(item_name)
+        item.location = self.get_location(location)
         if location not in self.disabled_locations:
             self.get_location(location).place_locked_item(item)
 
@@ -354,6 +432,14 @@ class TTYDWorld(World):
         if change:
             if item.name in stars:
                 state.prog_items[item.player]["stars"] += 1
+            for star in self.required_chapters:
+                if item.location is not None:
+                    if item.name == stars[star - 1] and self.options.star_shuffle == StarShuffle.option_vanilla:
+                        state.prog_items[item.player]["required_stars"] += 1
+                        break
+                    elif item.location.name == star_locations[star - 1] and self.options.star_shuffle == StarShuffle.option_stars_only:
+                        state.prog_items[item.player]["required_stars"] += 1
+                        break
         return change
 
     def remove(self, state: "CollectionState", item: "Item") -> bool:
@@ -361,6 +447,14 @@ class TTYDWorld(World):
         if change:
             if item.name in stars:
                 state.prog_items[item.player]["stars"] -= 1
+            for star in self.required_chapters:
+                if item.location is not None:
+                    if item.name == stars[star - 1] and self.options.star_shuffle == StarShuffle.option_vanilla:
+                        state.prog_items[item.player]["required_stars"] -= 1
+                        break
+                    elif item.location == star_locations[star - 1] and self.options.star_shuffle == StarShuffle.option_stars_only:
+                        state.prog_items[item.player]["required_stars"] -= 1
+                        break
         return change
 
     def generate_output(self, output_directory: str) -> None:
